package yoshi.editmenu;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;

import yoshi.editmenu.alpha.*;
import yoshi.editmenu.kernel.*;
import yoshi.editmenu.decrease.Decrease;

public class EditPanel extends ViewPanel
	implements MouseListener
{
	private ImageCache cache;
	private Rectangle selectArea;
	private boolean isSelectedArea = false;
	
	private JMenuItem initItem,undoItem,redoItem;
	
	private Frame parent;
	
	public EditPanel(Frame parent){
		super(parent);
		this.parent = parent;
		selectArea = new Rectangle();
		addMouseListener(this);
	}
	public void setCache(ImageCache cache){
		this.cache = cache;
		checkImage(cache.getImage());
	}
	protected void paintComponent(Graphics g){
		super.paintComponent(g);
		if(isSelectedArea){
			g.setColor(Color.white);
			g.drawRect(selectArea.x,selectArea.y,selectArea.width,selectArea.height);
		}
	}
	
	public JMenu createMenuCache(){
		//JMenu menu = new JMenu("LbV(C)");
		JMenu menu = new JMenu("LbV");
		//menu.setMnemonic('C');
		
		//initItem = new JMenuItem("(I)");
		initItem = new JMenuItem("");
		initItem.addActionListener(this);
		//initItem.setMnemonic('I');
		menu.add(initItem);
		
		//undoItem = new JMenuItem("蒼(U)");
		undoItem = new JMenuItem("蒼");
		undoItem.addActionListener(this);
		undoItem.setEnabled(false);
		//undoItem.setMnemonic('U');
		menu.add(undoItem);
		
		//redoItem = new JMenuItem("Ďs(R)");
		redoItem = new JMenuItem("Ďs");
		redoItem.addActionListener(this);
		redoItem.setEnabled(false);
		//redoItem.setMnemonic('R');
		menu.add(redoItem);
		
		return menu;
	}
	
	public JMenu createMenuEdit(){
		//JMenu menu = new JMenu("ҏW(E)");
		JMenu menu = new JMenu("ҏW");
		//menu.setMnemonic('E');
		
		//Reverce rev = new Reverce("RGB](T)");
		Reverce rev = new Reverce("RGB]");
		rev.addActionListener(this);
		//rev.setMnemonic('T');
		menu.add(rev);
		
		//Decrease decrease = new Decrease("F(D)");
		Decrease decrease = new Decrease("F");
		decrease.addActionListener(this);
		//decrease.setMnemonic('D');
		menu.add(decrease);
		
		menu.add(createMenuAlpha());
		menu.add(createMenuKernel());
		
		return menu;
	}
	private JMenu createMenuAlpha(){
		//JMenu menu = new JMenu("At@(A)");
		JMenu menu = new JMenu("At@");
		//menu.setMnemonic('A');
		
		//Blend blend = new Blend("At@lFω(B)");
		Blend blend = new Blend("At@lFω");
		blend.addActionListener(this);
		//blend.setMnemonic('B');
		menu.add(blend);
		
		//Alpha alpha = new Alpha("SAt@lω(A)");
		Alpha alpha = new Alpha("SAt@lω");
		//alpha.setMnemonic('A');
		alpha.addActionListener(this);
		menu.add(alpha);
		
		//ReverceAlpha realpha = new ReverceAlpha("At@l](R)");
		ReverceAlpha realpha = new ReverceAlpha("At@l]");
		realpha.addActionListener(this);
		//realpha.setMnemonic('R');
		menu.add(realpha);
		
		return menu;
	}
	private JMenu createMenuKernel(){
		//JMenu menu = new JMenu("ݍ݉Z(K)");
		JMenu menu = new JMenu("ݍ݉Z");
		//menu.setMnemonic('K');
		
		//Laplacian lap = new Laplacian("(L)");
		Laplacian lap = new Laplacian("");
		lap.addActionListener(this);
		//lap.setMnemonic('L');
		menu.add(lap);
		
		//Sharp sharp = new Sharp("N(S)");
		Sharp sharp = new Sharp("N");
		sharp.addActionListener(this);
		//sharp.setMnemonic('S');
		menu.add(sharp);
		
		//Average ave = new Average("R(A)");
		Average ave = new Average("R");
		ave.addActionListener(this);
		//ave.setMnemonic('A');
		menu.add(ave);
		
		//KernelItem kernel = new KernelItem("ݍ݉Z(K)");
		KernelItem kernel = new KernelItem("ݍ݉Z");
		kernel.addActionListener(this);
		//kernel.setMnemonic('K');
		menu.add(kernel);
		
		//menu.addSeparator();
		
		return menu;
	}
	
	public void actionPerformed(ActionEvent e){
		Object obj = e.getSource();
		if(obj instanceof EditMenuItem){
			if(getImage() != null){
				Rectangle oriRect = getImageRectangle(selectArea);
				BufferedImage img = ((EditMenuItem)obj).editImage(parent, getImage(), oriRect);
				if(img != null){
					cache.add(img);
					checkCache();
					checkImage(img);
				}
			}
		}else if(obj == initItem){
			cache.reset();
			checkCache();
			checkImage(cache.getImage());
		}else if(obj == undoItem){
			checkImage(cache.undo());
			checkCache();
		}else if(obj == redoItem){
			checkImage(cache.redo());
			checkCache();
		}else{
			super.actionPerformed(e);
		}
		repaint();
	}
	private void checkCache(){
		undoItem.setEnabled(cache.canUndo());
		redoItem.setEnabled(cache.canRedo());
	}
	public void mouseClicked(MouseEvent e){
		isSelectedArea = false;
		selectArea = new Rectangle(e.getX(),e.getY(),0,0);
		repaint();
	}
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mousePressed(MouseEvent e){
		isSelectedArea = false;
		selectArea = new Rectangle(e.getX(),e.getY(),0,0);
		repaint();
	}
	
	public void mouseReleased(MouseEvent e){
		setArea(e.getPoint());
		repaint();
	}
	
	private void setArea(Point p){
		isSelectedArea = true;
		if(selectArea.x <= p.x){
			selectArea.width = p.x - selectArea.x;
		}else{
			selectArea.x = p.x;
			selectArea.width = selectArea.x - p.x;
		}
		if(selectArea.y <= p.y){
			selectArea.height = p.y - selectArea.y;
		}else{
			selectArea.y = p.y;
			selectArea.height = selectArea.y - p.y;
		}
	}
}
