package yoshi.editmenu.kernel;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.image.*;

public class KernelDialog extends JDialog
	implements ActionListener,ListSelectionListener
{
	private JTabbedPane tab;
	private JList list;
	private String[] strKernels;
	private Kernel[] kernels;
	
	private JTextField[] tfdKernels;
	
	private JButton btnOk,btnCancel;
	private JCheckBox chkAlpha,chkEdge;
	
	private boolean isSelected = false;
	
	public KernelDialog(Frame parent){
		super(parent,true);
		initComponents();
		//pack();
		setLocationRelativeTo(parent);
	}
	public ConvolveOp getConvolveOp(){
		Kernel kernel = getKernel();
		if(kernel != null){
			int mode = chkEdge.isSelected() ? ConvolveOp.EDGE_NO_OP : ConvolveOp.EDGE_ZERO_FILL;
			return new ConvolveOp(kernel,mode,null);
		}else{
			return null;
		}
	}
	public Kernel getKernel(){
		int index = tab.getSelectedIndex();
		switch(index){
		case 0:
			index = list.getSelectedIndex();
			if(index >= 0){
				return kernels[index];
			}else{
				return null;
			}
		case 1:
			return null;
		}
		return null;
	}
	
	public boolean isSelected(){
		return isSelected;
	}
	
	public boolean isCheckAlpha(){
		return !chkAlpha.isSelected();
	}
	private void initComponents(){
		setSize(300,200);
		setTitle("ݍ݉Z");
		
		getContentPane().setLayout(new BorderLayout());
		
		tab = new JTabbedPane();
		
		initKernel();
		list = new JList(strKernels);
		list.setSelectedIndex(0);
		list.addListSelectionListener(this);
		tab.add(list,"ȒPݒ");
		
		JPanel kernelView = new JPanel();
		kernelView.setLayout(new GridLayout(3,3));
		tfdKernels = new JTextField[9];
		for(int i=0;i<9;i++){
			tfdKernels[i] = new JTextField(String.valueOf(kernels[0].getKernelData(null)[i]));
			kernelView.add(tfdKernels[i]);
		}
		tab.add(kernelView,"ڍאݒ");
		
		getContentPane().add(tab,BorderLayout.CENTER);
		getContentPane().add(createSouth(),BorderLayout.SOUTH);
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				isSelected = false;
				setVisible(false);
			}
		});
	}
	
	private void initKernel(){
		strKernels = new String[5];
		kernels = new Kernel[5];
		
		strKernels[0] = "GbWo (vVA)";
		kernels[0] = new Kernel(3,3,new float[]{0, 1, 0, 1, -4, 1, 0, 1, 0});
		
		strKernels[1] = "GbW (GbW)";
		kernels[1] = new Kernel(3,3,new float[]{0, -1, 0, -1, 5, -1, 0, -1, 0});

		strKernels[2] = "GbW (V[y)";
		kernels[2] = new Kernel(3,3,new float[]{-1, -1, -1, -1, 9, -1, -1, -1, -1});
		
		strKernels[3] = "ڂ (R)";
		float n = (float)(1.0/9.0);
		kernels[3] = new Kernel(3,3,new float[]{n, n, n, n, n, n, n, n, n});
		
		strKernels[4] = "ڂ (u[)";
		kernels[4] = new Kernel(3,3,new float[]{0.1f, 0.1f, 0.1f, 0.1f, 0.2f, 0.1f, 0.1f, 0.1f, 0.1f});
		
	}
	private JComponent createSouth(){
		JPanel p = new JPanel();
		p.setLayout(new GridLayout(2,2));
		
		chkAlpha = new JCheckBox("At@vZ");
		p.add(chkAlpha);
		
		btnOk = new JButton("ϊ");
		btnOk.addActionListener(this);
		p.add(btnOk);
		
		chkEdge = new JCheckBox("̂");
		p.add(chkEdge);
		
		btnCancel = new JButton("");
		btnCancel.addActionListener(this);
		p.add(btnCancel);
		
		return p;
	}
	
	public void actionPerformed(ActionEvent e){
		Object obj = e.getSource();
		if(obj == btnOk){
			isSelected = true;
			setVisible(false);
		}else if(obj == btnCancel){
			isSelected = false;
			setVisible(false);
		}
	}
	public void valueChanged(ListSelectionEvent e)  {
		if(e.getSource() == list){
			int index = list.getSelectedIndex();
			for(int i=0;i<9;i++){
				tfdKernels[i].setText(String.valueOf(kernels[index].getKernelData(null)[i]));
			}
		}
	}
}
