package yoshi.imageio;

import javax.swing.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import yoshi.imageio.arcivefilter.*;
import yoshi.imageio.arciver.*;

public class ArciveChooser extends JDialog
	implements ActionListener,PropertyChangeListener 
{
	private JFileChooser chooser;
	private boolean isSelectedFile = false;
	private Frame parent;
	private ImageArciveFilterFactory factory;
	
	public ArciveChooser(Frame parent,ImageArciveFilterFactory factory){
		this(parent,factory,new File("."));
	}
	public ArciveChooser(Frame parent,ImageArciveFilterFactory factory,File current){
		super(parent,true);
		this.parent = parent;
		this.factory = factory;
		chooser = new JFileChooser(current);
		chooser.addPropertyChangeListener(this);
		chooser.addActionListener(this);
		chooser.setAcceptAllFileFilterUsed(false);
		
		ImageArciveFilter[] filters = factory.getImageArciveFilters();
		for(int i=0;i<filters.length;i++){
			chooser.addChoosableFileFilter(filters[i]);
		}
		chooser.setFileFilter(filters[0]);
		chooser.setFileSelectionMode(filters[0].getFileSelectionMode());
		
		setTitle("ArciveChooser");
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(chooser,BorderLayout.CENTER);
		
		pack();
		setLocationRelativeTo(parent);
	}
	public boolean isSelected(){
		return isSelectedFile;
	}
	public ImageArciver getImageArciver(){
		if(isSelectedFile){
			ImageArciveFilter filter = (ImageArciveFilter)chooser.getFileFilter();
			ImageArciver arc = filter.createImageArciver(chooser.getSelectedFile());
			if(arc == null){
				filter = factory.getImageArciveFilter(parent,chooser.getSelectedFile());
				arc = filter.createImageArciver(chooser.getSelectedFile());
			}
			return arc;
		}else{
			return null;
		}
	}
	public void propertyChange(PropertyChangeEvent e) {
		String prop = e.getPropertyName();
		if(prop.equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY)) {
			ImageArciveFilter filter = (ImageArciveFilter)chooser.getFileFilter();
			chooser.setFileSelectionMode(filter.getFileSelectionMode());
		}
		chooser.repaint();
	}
	public void updateUI(){
		SwingUtilities.updateComponentTreeUI(chooser);
	}
	public void actionPerformed(ActionEvent e){
		String cmd = e.getActionCommand();
		
		if(cmd.equals(JFileChooser.APPROVE_SELECTION)){
			if(chooser.getSelectedFile() != null){
				isSelectedFile = true;
				setVisible(false);
			}
		}else if(cmd.equals(JFileChooser.CANCEL_SELECTION )){
			isSelectedFile = false;
			setVisible(false);
		}
	}
}
