package yoshi.imageio;

import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.Frame;
import javax.swing.*;

import yoshi.imageio.filefilter.*;
//import yoshi.imageio.reader.SPIFileFilterFactory;

public class ImageFileFilterFactory
{
	private ImageFileFilter[] readFilters;
	private ImageFileFilter[] writeFilters;
	
	public ImageFileFilterFactory(){
		Vector readList = new Vector();
		Vector writeList = new Vector();
		setDefaultFileFilter(readList,writeList);
		setPluginFilter(readList,writeList);
		//setSPIFileFilter(readList);
		
		readFilters = new ImageFileFilter[readList.size()];
		readList.copyInto(readFilters);
		
		writeFilters = new ImageFileFilter[writeList.size()];
		writeList.copyInto(writeFilters);
		
	}
	
	public ImageFileFilter[] getReaderImageFileFilters(){
		return readFilters;
	}
	public ImageFileFilter[] getWriterImageFileFilters(){
		return writeFilters;
	}
	public ImageFileFilter getReaderImageFileFilter(Frame parent,File file){
		if(file.isFile()){
			Vector v = new Vector();
			for(int i=0;i<readFilters.length;i++){
				if(readFilters[i].accept(file) && readFilters[i].canRead()){
					v.add(readFilters[i]);
				}
			}
			if(v.size() > 1){
				JList list = new JList(v);
				list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
				list.setSelectedIndex(0);
				int result = JOptionPane.showConfirmDialog(parent,list,"tB^̑I",
					JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE);
				if(result ==  JOptionPane.OK_OPTION ){
					return (ImageFileFilter)list.getSelectedValue();
				}else{
					return null;
				}
			}else if(v.size() == 1){
				return (ImageFileFilter)v.get(0);
			}else{
				return null;
			}
		}else{
			return null;
		}
	}
	
	private void setDefaultFileFilter(List readList,List writeList){
		readList.add(new AllFileFilter());
		readList.add(new BMPFileFilter());
		readList.add(new JPGFileFilter());
		readList.add(new PNGFileFilter());
		readList.add(new PDTFileFilter());
		//readList.add(new JavaFileFilter());
		
		writeList.add(new BMPFileFilter());
		writeList.add(new JPGFileFilter());
		writeList.add(new PNGFileFilter());
	}
	
	private void setPluginFilter(List readList,List writeList){
		File dir = ((new File(".")).getAbsoluteFile()).getParentFile();
		URLClassLoader loader = null;
		try{
			loader = new URLClassLoader(new URL[]{dir.toURL()});
		}catch(MalformedURLException e){
			System.out.println(e);
			return;
		}
		File[] list = dir.listFiles(new FileFilterClass());
		if(list != null){
			Vector v = new Vector();
			for(int i=0;i<list.length;i++){
				Class c = createClass(list[i],loader);
				if(c != null){
					try{
						v.add((ImageFileFilter)c.newInstance());
					}catch(Throwable t){
						
					}
				}
			}
			ImageFileFilter[] filter = new ImageFileFilter[v.size()];
			v.copyInto(filter);
			
			for(int i=0;i<filter.length;i++){
				if(filter[i].canRead()){
					readList.add(filter[i]);
				}
				if(filter[i].canWrite()){
					readList.add(filter[i]);
				}
			}
		}
	}
	
/*	private void setSPIFileFilter(List readList){
		if(SPIFileFilterFactory.canUse()){
			ImageFileFilter[] filter = (new SPIFileFilterFactory()).getFileFilter();
			for(int i=0;i<filter.length;i++){
				readList.add(filter[i]);
			}
		}
	}*/
	
	public Class createClass(File file,ClassLoader loader){
		String name = file.getName();
		int n = name.lastIndexOf('.');
		name = name.substring(0,n);
		try{
			return loader.loadClass(name);
		}catch(ClassNotFoundException e){
			System.out.println(e);
			return null;
		}
	}
	public void updateUI(){
		for(int i=0;i<readFilters.length;i++){
			readFilters[i].updateUI();
		}
		for(int i=0;i<writeFilters.length;i++){
			writeFilters[i].updateUI();
		}
	}
	private class FileFilterClass implements FileFilter
	{
		public boolean accept(File pathname){
			String name = pathname.getName();
			return name.endsWith("FileFilter.class");
		}
	}
}
