package yoshi.imageio.filefilter;

import yoshi.imageio.reader.BMPReader;
import yoshi.imageio.writer.BMPWriter;
import yoshi.imageio.filedialog.BMPDialog;

import javax.swing.SwingUtilities;
import java.io.*;
import java.awt.*;
import java.awt.image.BufferedImage;

public class BMPFileFilter extends ImageFileFilter
{
	private static final String[] exts = new String[]{"bmp"};
	private static final String description = "Windows Bitmap File (bmp)";
	
	private BMPWriter writer;
	private BMPDialog dialog;
	private Frame parent;
	
	public boolean canRead(){
		return true;
	}
	public BufferedImage read(Frame parent,InputStream in) throws IOException{
		return (new BMPReader(in)).getImage();
	}
	public boolean canWrite(){
		return true;
	}
	
	public boolean setWriteParam(Frame parent, BufferedImage img, boolean isShowDialog){
		if(img == null || parent == null) return false;
		
		if(this.parent == null || this.parent != parent){
			dialog = new BMPDialog(parent);
			this.parent = parent;
		}
		
		if(writer == null) writer = new BMPWriter();
		writer.setImage(img);
		
		if(isShowDialog){
			dialog.setColorNumber(writer.getColorNumber());
			dialog.setVisible(true);
			if(!dialog.isWrite()) return false;
		}
		
		writer.setBitSize(dialog.getBitSize());
		return true;
	}
	public void write(OutputStream out) throws IOException{
		if(writer != null){
			writer.write(out);
		}else{
			throw new IOException("Internal Error");
		}
	}
	
	public String[] getExtentions(){
		return exts;
	}
	public String getDescription(){
		return description;
	}
	public void updateUI(){
		if(dialog != null) SwingUtilities.updateComponentTreeUI(dialog);
	}
}