package yoshi.imageio.filefilter;

import yoshi.imageio.filedialog.JPGDialog;

import java.io.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import javax.swing.SwingUtilities;
import com.sun.image.codec.jpeg.*;

public class JPGFileFilter extends ImageFileFilter
{
	private static final String[] exts = new String[]{"jpg","jpeg"};
	private static final String description = "JPEG/JFIF (jpg, jpeg)";
	
	private JPEGEncodeParam param;
	private BufferedImage img;
	private JPGDialog dialog;
	private Frame parent;
	
	public boolean canRead(){
		return true;
	}
	public BufferedImage read(Frame parent,InputStream in) throws IOException{
		JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
		try{
			return decoder.decodeAsBufferedImage();
		}catch(Exception e){
			throw new IOException(e.toString());
		}
	}
	
	public boolean canWrite(){
		return true;
	}
	public boolean setWriteParam(Frame parent,BufferedImage img, boolean isShowDialog){
		if(img == null || parent == null) return false;
		
		if(this.parent == null || this.parent != parent){
			dialog = new JPGDialog(parent);
			this.parent = parent;
		}
		
		if(isShowDialog){
			dialog.setVisible(true);
			if(!dialog.isWrite()) return false;
		}
		param = JPEGCodec.getDefaultJPEGEncodeParam(img);
		param.setQuality(dialog.getQuality(),false);
		this.img = img;
		return true;
	}
	public void write(OutputStream out) throws IOException{
		if(param != null){
			JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out,param);
			try {
				encoder.encode(img);
			}catch(IOException ie){
				throw ie;
			}catch(Exception e){
				throw new IOException(e.toString());
			}
			//System.out.println("write");
			encoder = null;
			param = null;
			img = null;
		}else{
			throw new IOException("Internal Error");
		}
	}
	public String[] getExtentions(){
		return exts;
	}
	public String getDescription(){
		return description;
	}
	public void updateUI(){
		if(dialog != null) SwingUtilities.updateComponentTreeUI(dialog);
	}
}