package yoshi.imageio.filefilter;

import java.io.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import javax.swing.SwingUtilities;

import yoshi.imageio.filedialog.PNGDialog;
import yoshi.imageio.reader.PNGReader;
import yoshi.imageio.writer.PNGWriter;

public class PNGFileFilter extends ImageFileFilter
{
	private static final String[] exts = new String[]{"png"};
	private static final String description = "Portable Network Graphics (png)";
	
	private PNGWriter writer;
	private PNGDialog dialog;
	private Frame parent;
	
	public boolean canRead(){
		return true;
	}
	public BufferedImage read(Frame parent,InputStream in) throws IOException{
		return (new PNGReader(in)).getImage();
	}
	public boolean canWrite(){
		return true;
	}
	public boolean setWriteParam(Frame parent,BufferedImage img, boolean isShowDialog){
		if(img == null || parent == null) return false;
		
		if(this.parent == null || this.parent != parent){
			dialog = new PNGDialog(parent);
			this.parent = parent;
		}
		
		if(writer == null) writer = new PNGWriter();
		writer.setImage(img);
		
		if(isShowDialog){
			dialog.setColorNumber(writer.getColorNumber());
			dialog.setVisible(true);
			if(!dialog.isWrite()) return false;
		}
		writer.setFilterLevel(dialog.getFilterLevel());
		writer.setCompressionLevel(dialog.getCompressionLevel());
		writer.setUseAlpha(dialog.getUseAlpha());
		writer.setUseGray(dialog.getUseGray());
		writer.setUsePalette(dialog.getUsePalette());
		writer.setUseInterlace(dialog.getUseInterlace());
		return true;
	}
	public void write(OutputStream out) throws IOException{
		if(writer != null){
			writer.write(out);
		}else{
			throw new IOException("Internal Error");
		}
	}
	public String[] getExtentions(){
		return exts;
	}
	public String getDescription(){
		return description;
	}
	public void updateUI(){
		if(dialog != null) SwingUtilities.updateComponentTreeUI(dialog);
	}
}